@@ -2,12 +2,17 @@
 Utility functions for difference-in-differences estimation.
 """
 
-from typing import Any, Dict, List, Optional, Tuple, Union
+import warnings
+from dataclasses import dataclass, field
+from typing import Any, Dict, List, Optional, Tuple
 
 import numpy as np
 import pandas as pd
 from scipy import stats
 
+from diff_diff.linalg import compute_robust_vcov as _compute_robust_vcov_linalg
+from diff_diff.linalg import solve_ols as _solve_ols_linalg
+
 # Numerical constants for optimization algorithms
 _OPTIMIZATION_MAX_ITER = 1000  # Maximum iterations for weight optimization
 _OPTIMIZATION_TOL = 1e-8  # Convergence tolerance for optimization
@@ -41,11 +46,14 @@ def validate_binary(arr: np.ndarray, name: str) -> None:
 def compute_robust_se(
     X: np.ndarray,
     residuals: np.ndarray,
-    cluster_ids: np.ndarray = None
+    cluster_ids: Optional[np.ndarray] = None
 ) -> np.ndarray:
     """
     Compute heteroskedasticity-robust (HC1) or cluster-robust standard errors.
 
+    This function is a thin wrapper around the optimized implementation in
+    diff_diff.linalg for backwards compatibility.
+
     Parameters
     ----------
     X : np.ndarray
@@ -60,55 +68,15 @@ def compute_robust_se(
     np.ndarray
         Variance-covariance matrix of shape (k, k).
     """
-    n, k = X.shape
-    XtX = X.T @ X
-
-    if cluster_ids is None:
-        # HC1 robust standard errors
-        # HC1 adjustment factor: n / (n - k)
-        adjustment = n / (n - k)
-
-        # Create diagonal matrix with squared residuals
-        u_squared = residuals ** 2
-
-        # Meat of the sandwich: X' * diag(u^2) * X
-        meat = X.T @ (X * u_squared[:, np.newaxis])
-
-        # Compute XtX^{-1} @ meat @ XtX^{-1} using solve() for numerical stability
-        # First solve XtX @ temp = meat to get temp = XtX^{-1} @ meat
-        temp = np.linalg.solve(XtX, meat)
-        # Then solve XtX @ vcov = temp.T and transpose to get XtX^{-1} @ meat @ XtX^{-1}
-        vcov = adjustment * np.linalg.solve(XtX, temp.T).T
-    else:
-        # Cluster-robust standard errors
-        unique_clusters = np.unique(cluster_ids)
-        n_clusters = len(unique_clusters)
-
-        # Adjustment factor for cluster-robust SEs
-        adjustment = (n_clusters / (n_clusters - 1)) * ((n - 1) / (n - k))
-
-        # Compute the meat of the sandwich
-        meat = np.zeros((k, k))
-        for cluster in unique_clusters:
-            mask = cluster_ids == cluster
-            X_c = X[mask]
-            u_c = residuals[mask]
-            score_c = X_c.T @ u_c
-            meat += np.outer(score_c, score_c)
-
-        # Compute XtX^{-1} @ meat @ XtX^{-1} using solve() for numerical stability
-        temp = np.linalg.solve(XtX, meat)
-        vcov = adjustment * np.linalg.solve(XtX, temp.T).T
-
-    return vcov
+    return _compute_robust_vcov_linalg(X, residuals, cluster_ids)
 
 
 def compute_confidence_interval(
     estimate: float,
     se: float,
     alpha: float = 0.05,
-    df: int = None
-) -> tuple:
+    df: Optional[int] = None
+) -> Tuple[float, float]:
     """
     Compute confidence interval for an estimate.
 
@@ -139,7 +107,7 @@ def compute_confidence_interval(
     return (lower, upper)
 
 
-def compute_p_value(t_stat: float, df: int = None, two_sided: bool = True) -> float:
+def compute_p_value(t_stat: float, df: Optional[int] = None, two_sided: bool = True) -> float:
     """
     Compute p-value for a t-statistic.
 
@@ -165,7 +133,381 @@ def compute_p_value(t_stat: float, df: int = None, two_sided: bool = True) -> fl
     if two_sided:
         p_value *= 2
 
-    return p_value
+    return float(p_value)
+
+
+# =============================================================================
+# Wild Cluster Bootstrap
+# =============================================================================
+
+
+@dataclass
+class WildBootstrapResults:
+    """
+    Results from wild cluster bootstrap inference.
+
+    Attributes
+    ----------
+    se : float
+        Bootstrap standard error of the coefficient.
+    p_value : float
+        Bootstrap p-value (two-sided).
+    t_stat_original : float
+        Original t-statistic from the data.
+    ci_lower : float
+        Lower bound of the confidence interval.
+    ci_upper : float
+        Upper bound of the confidence interval.
+    n_clusters : int
+        Number of clusters in the data.
+    n_bootstrap : int
+        Number of bootstrap replications.
+    weight_type : str
+        Type of bootstrap weights used ("rademacher", "webb", or "mammen").
+    alpha : float
+        Significance level used for confidence interval.
+    bootstrap_distribution : np.ndarray, optional
+        Full bootstrap distribution of coefficients (if requested).
+
+    References
+    ----------
+    Cameron, A. C., Gelbach, J. B., & Miller, D. L. (2008).
+    Bootstrap-Based Improvements for Inference with Clustered Errors.
+    The Review of Economics and Statistics, 90(3), 414-427.
+    """
+
+    se: float
+    p_value: float
+    t_stat_original: float
+    ci_lower: float
+    ci_upper: float
+    n_clusters: int
+    n_bootstrap: int
+    weight_type: str
+    alpha: float = 0.05
+    bootstrap_distribution: Optional[np.ndarray] = field(default=None, repr=False)
+
+    def summary(self) -> str:
+        """Generate formatted summary of bootstrap results."""
+        lines = [
+            "Wild Cluster Bootstrap Results",
+            "=" * 40,
+            f"Bootstrap SE:        {self.se:.6f}",
+            f"Bootstrap p-value:   {self.p_value:.4f}",
+            f"Original t-stat:     {self.t_stat_original:.4f}",
+            f"CI ({int((1-self.alpha)*100)}%):           [{self.ci_lower:.6f}, {self.ci_upper:.6f}]",
+            f"Number of clusters:  {self.n_clusters}",
+            f"Bootstrap reps:      {self.n_bootstrap}",
+            f"Weight type:         {self.weight_type}",
+        ]
+        return "\n".join(lines)
+
+    def print_summary(self) -> None:
+        """Print formatted summary to stdout."""
+        print(self.summary())
+
+
+def _generate_rademacher_weights(n_clusters: int, rng: np.random.Generator) -> np.ndarray:
+    """
+    Generate Rademacher weights: +1 or -1 with probability 0.5.
+
+    Parameters
+    ----------
+    n_clusters : int
+        Number of clusters.
+    rng : np.random.Generator
+        Random number generator.
+
+    Returns
+    -------
+    np.ndarray
+        Array of Rademacher weights.
+    """
+    return np.asarray(rng.choice([-1.0, 1.0], size=n_clusters))
+
+
+def _generate_webb_weights(n_clusters: int, rng: np.random.Generator) -> np.ndarray:
+    """
+    Generate Webb's 6-point distribution weights.
+
+    Values: {-sqrt(3/2), -sqrt(2/2), -sqrt(1/2), sqrt(1/2), sqrt(2/2), sqrt(3/2)}
+    with probabilities proportional to {1, 2, 3, 3, 2, 1}.
+
+    This distribution is recommended for very few clusters (G < 10) as it
+    provides better finite-sample properties than Rademacher weights.
+
+    Parameters
+    ----------
+    n_clusters : int
+        Number of clusters.
+    rng : np.random.Generator
+        Random number generator.
+
+    Returns
+    -------
+    np.ndarray
+        Array of Webb weights.
+
+    References
+    ----------
+    Webb, M. D. (2014). Reworking wild bootstrap based inference for
+    clustered errors. Queen's Economics Department Working Paper No. 1315.
+    """
+    values = np.array([
+        -np.sqrt(3 / 2), -np.sqrt(2 / 2), -np.sqrt(1 / 2),
+        np.sqrt(1 / 2), np.sqrt(2 / 2), np.sqrt(3 / 2)
+    ])
+    probs = np.array([1, 2, 3, 3, 2, 1]) / 12
+    return np.asarray(rng.choice(values, size=n_clusters, p=probs))
+
+
+def _generate_mammen_weights(n_clusters: int, rng: np.random.Generator) -> np.ndarray:
+    """
+    Generate Mammen's two-point distribution weights.
+
+    Values: {-(sqrt(5)-1)/2, (sqrt(5)+1)/2}
+    with probabilities {(sqrt(5)+1)/(2*sqrt(5)), (sqrt(5)-1)/(2*sqrt(5))}.
+
+    This distribution satisfies E[v]=0, E[v^2]=1, E[v^3]=1, which provides
+    asymptotic refinement for skewed error distributions.
+
+    Parameters
+    ----------
+    n_clusters : int
+        Number of clusters.
+    rng : np.random.Generator
+        Random number generator.
+
+    Returns
+    -------
+    np.ndarray
+        Array of Mammen weights.
+
+    References
+    ----------
+    Mammen, E. (1993). Bootstrap and Wild Bootstrap for High Dimensional
+    Linear Models. The Annals of Statistics, 21(1), 255-285.
+    """
+    sqrt5 = np.sqrt(5)
+    # Values from Mammen (1993)
+    val1 = -(sqrt5 - 1) / 2  # approximately -0.618
+    val2 = (sqrt5 + 1) / 2   # approximately 1.618 (golden ratio)
+
+    # Probability of val1
+    p1 = (sqrt5 + 1) / (2 * sqrt5)  # approximately 0.724
+
+    return np.asarray(rng.choice([val1, val2], size=n_clusters, p=[p1, 1 - p1]))
+
+
+def wild_bootstrap_se(
+    X: np.ndarray,
+    y: np.ndarray,
+    residuals: np.ndarray,
+    cluster_ids: np.ndarray,
+    coefficient_index: int,
+    n_bootstrap: int = 999,
+    weight_type: str = "rademacher",
+    null_hypothesis: float = 0.0,
+    alpha: float = 0.05,
+    seed: Optional[int] = None,
+    return_distribution: bool = False
+) -> WildBootstrapResults:
+    """
+    Compute wild cluster bootstrap standard errors and p-values.
+
+    Implements the Wild Cluster Residual (WCR) bootstrap procedure from
+    Cameron, Gelbach, and Miller (2008). Uses the restricted residuals
+    approach (imposing H0: coefficient = null_hypothesis) for more accurate
+    p-value computation.
+
+    Parameters
+    ----------
+    X : np.ndarray
+        Design matrix of shape (n, k).
+    y : np.ndarray
+        Outcome vector of shape (n,).
+    residuals : np.ndarray
+        OLS residuals from unrestricted regression, shape (n,).
+    cluster_ids : np.ndarray
+        Cluster identifiers of shape (n,).
+    coefficient_index : int
+        Index of the coefficient for which to compute bootstrap inference.
+        For DiD, this is typically 3 (the treatment*post interaction term).
+    n_bootstrap : int, default=999
+        Number of bootstrap replications. Odd numbers are recommended for
+        exact p-value computation.
+    weight_type : str, default="rademacher"
+        Type of bootstrap weights:
+        - "rademacher": +1 or -1 with equal probability (standard choice)
+        - "webb": 6-point distribution (recommended for <10 clusters)
+        - "mammen": Two-point distribution with skewness correction
+    null_hypothesis : float, default=0.0
+        Value of the null hypothesis for p-value computation.
+    alpha : float, default=0.05
+        Significance level for confidence interval.
+    seed : int, optional
+        Random seed for reproducibility. If None (default), results
+        will vary between runs.
+    return_distribution : bool, default=False
+        If True, include full bootstrap distribution in results.
+
+    Returns
+    -------
+    WildBootstrapResults
+        Dataclass containing bootstrap SE, p-value, confidence interval,
+        and other inference results.
+
+    Raises
+    ------
+    ValueError
+        If weight_type is not recognized or if there are fewer than 2 clusters.
+
+    Warns
+    -----
+    UserWarning
+        If the number of clusters is less than 5, as bootstrap inference
+        may be unreliable.
+
+    Examples
+    --------
+    >>> from diff_diff.utils import wild_bootstrap_se
+    >>> results = wild_bootstrap_se(
+    ...     X, y, residuals, cluster_ids,
+    ...     coefficient_index=3,  # ATT coefficient
+    ...     n_bootstrap=999,
+    ...     weight_type="rademacher",
+    ...     seed=42
+    ... )
+    >>> print(f"Bootstrap SE: {results.se:.4f}")
+    >>> print(f"Bootstrap p-value: {results.p_value:.4f}")
+
+    References
+    ----------
+    Cameron, A. C., Gelbach, J. B., & Miller, D. L. (2008).
+    Bootstrap-Based Improvements for Inference with Clustered Errors.
+    The Review of Economics and Statistics, 90(3), 414-427.
+
+    MacKinnon, J. G., & Webb, M. D. (2018). The wild bootstrap for
+    few (treated) clusters. The Econometrics Journal, 21(2), 114-135.
+    """
+    # Validate inputs
+    valid_weight_types = ["rademacher", "webb", "mammen"]
+    if weight_type not in valid_weight_types:
+        raise ValueError(
+            f"weight_type must be one of {valid_weight_types}, got '{weight_type}'"
+        )
+
+    unique_clusters = np.unique(cluster_ids)
+    n_clusters = len(unique_clusters)
+
+    if n_clusters < 2:
+        raise ValueError(
+            f"Wild cluster bootstrap requires at least 2 clusters, got {n_clusters}"
+        )
+
+    if n_clusters < 5:
+        warnings.warn(
+            f"Only {n_clusters} clusters detected. Wild bootstrap inference may be "
+            "unreliable with fewer than 5 clusters. Consider using Webb weights "
+            "(weight_type='webb') for improved finite-sample properties.",
+            UserWarning
+        )
+
+    # Initialize RNG
+    rng = np.random.default_rng(seed)
+
+    # Select weight generator
+    weight_generators = {
+        "rademacher": _generate_rademacher_weights,
+        "webb": _generate_webb_weights,
+        "mammen": _generate_mammen_weights,
+    }
+    generate_weights = weight_generators[weight_type]
+
+    n = X.shape[0]
+
+    # Step 1: Compute original coefficient and cluster-robust SE
+    beta_hat, _, vcov_original = _solve_ols_linalg(
+        X, y, cluster_ids=cluster_ids, return_vcov=True
+    )
+    original_coef = beta_hat[coefficient_index]
+    se_original = np.sqrt(vcov_original[coefficient_index, coefficient_index])
+    t_stat_original = (original_coef - null_hypothesis) / se_original
+
+    # Step 2: Impose null hypothesis (restricted estimation)
+    # Create restricted y: y_restricted = y - X[:, coef_index] * null_hypothesis
+    # This imposes the null that the coefficient equals null_hypothesis
+    y_restricted = y - X[:, coefficient_index] * null_hypothesis
+
+    # Fit restricted model (but we need to drop the column for the restricted coef)
+    # Actually, for WCR bootstrap we keep all columns but impose the null via residuals
+    # Re-estimate with the restricted dependent variable
+    beta_restricted, residuals_restricted, _ = _solve_ols_linalg(
+        X, y_restricted, return_vcov=False
+    )
+
+    # Create cluster-to-observation mapping for efficiency
+    cluster_map = {c: np.where(cluster_ids == c)[0] for c in unique_clusters}
+    cluster_indices = [cluster_map[c] for c in unique_clusters]
+
+    # Step 3: Bootstrap loop
+    bootstrap_t_stats = np.zeros(n_bootstrap)
+    bootstrap_coefs = np.zeros(n_bootstrap)
+
+    for b in range(n_bootstrap):
+        # Generate cluster-level weights
+        cluster_weights = generate_weights(n_clusters, rng)
+
+        # Map cluster weights to observations
+        obs_weights = np.zeros(n)
+        for g, indices in enumerate(cluster_indices):
+            obs_weights[indices] = cluster_weights[g]
+
+        # Construct bootstrap sample: y* = X @ beta_restricted + e_restricted * weights
+        y_star = X @ beta_restricted + residuals_restricted * obs_weights
+
+        # Estimate bootstrap coefficients with cluster-robust SE
+        beta_star, residuals_star, vcov_star = _solve_ols_linalg(
+            X, y_star, cluster_ids=cluster_ids, return_vcov=True
+        )
+        bootstrap_coefs[b] = beta_star[coefficient_index]
+        se_star = np.sqrt(vcov_star[coefficient_index, coefficient_index])
+
+        # Compute bootstrap t-statistic (under null hypothesis)
+        if se_star > 0:
+            bootstrap_t_stats[b] = (beta_star[coefficient_index] - null_hypothesis) / se_star
+        else:
+            bootstrap_t_stats[b] = 0.0
+
+    # Step 4: Compute bootstrap p-value
+    # P-value is proportion of |t*| >= |t_original|
+    p_value = np.mean(np.abs(bootstrap_t_stats) >= np.abs(t_stat_original))
+
+    # Ensure p-value is at least 1/(n_bootstrap+1) to avoid exact zero
+    p_value = float(max(float(p_value), 1 / (n_bootstrap + 1)))
+
+    # Step 5: Compute bootstrap SE and confidence interval
+    # SE from standard deviation of bootstrap coefficient distribution
+    se_bootstrap = float(np.std(bootstrap_coefs, ddof=1))
+
+    # Percentile confidence interval from bootstrap distribution
+    lower_percentile = alpha / 2 * 100
+    upper_percentile = (1 - alpha / 2) * 100
+    ci_lower = float(np.percentile(bootstrap_coefs, lower_percentile))
+    ci_upper = float(np.percentile(bootstrap_coefs, upper_percentile))
+
+    return WildBootstrapResults(
+        se=se_bootstrap,
+        p_value=p_value,
+        t_stat_original=t_stat_original,
+        ci_lower=ci_lower,
+        ci_upper=ci_upper,
+        n_clusters=n_clusters,
+        n_bootstrap=n_bootstrap,
+        weight_type=weight_type,
+        alpha=alpha,
+        bootstrap_distribution=bootstrap_coefs if return_distribution else None
+    )
 
 
 def check_parallel_trends(
@@ -212,7 +554,7 @@ def check_parallel_trends(
     control_data = pre_data[pre_data[treatment_group] == 0]
 
     # Simple linear regression for trends
-    def compute_trend(group_data):
+    def compute_trend(group_data: pd.DataFrame) -> Tuple[float, float]:
         time_values = group_data[time].values
         outcome_values = group_data[outcome].values
 
@@ -468,8 +810,6 @@ def _compute_outcome_changes(
         ]["_outcome_change"].values
     else:
         # Aggregate changes: compute mean change per period per group
-        periods = sorted(data[time].unique())
-
         treated_data = data[data[treatment_group] == 1]
         control_data = data[data[treatment_group] == 0]
 
@@ -688,10 +1028,10 @@ def compute_synthetic_weights(
     n_pre, n_control = Y_control.shape
 
     if n_control == 0:
-        return np.array([])
+        return np.asarray([])
 
     if n_control == 1:
-        return np.array([1.0])
+        return np.asarray([1.0])
 
     # Initialize with uniform weights
     weights = np.ones(n_control) / n_control
@@ -733,7 +1073,7 @@ def compute_synthetic_weights(
         # Fallback to uniform if all weights are zeroed
         weights = np.ones(n_control) / n_control
 
-    return weights
+    return np.asarray(weights)
 
 
 def _project_simplex(v: np.ndarray) -> np.ndarray:
@@ -771,7 +1111,7 @@ def _project_simplex(v: np.ndarray) -> np.ndarray:
 
     theta = (cssv[rho_val] - 1) / (rho_val + 1)
 
-    return np.maximum(v - theta, 0)
+    return np.asarray(np.maximum(v - theta, 0))
 
 
 def compute_time_weights(
@@ -809,7 +1149,7 @@ def compute_time_weights(
     n_pre = len(Y_treated)
 
     if n_pre <= 1:
-        return np.ones(n_pre)
+        return np.asarray(np.ones(n_pre))
 
     # Compute mean control outcomes per period
     control_means = np.mean(Y_control, axis=1)
@@ -824,7 +1164,7 @@ def compute_time_weights(
     # Normalize to sum to 1
     weights = inv_diffs / np.sum(inv_diffs)
 
-    return weights
+    return np.asarray(weights)
 
 
 def compute_sdid_estimator(
@@ -887,7 +1227,7 @@ def compute_sdid_estimator(
     # SDID estimator
     tau = did_treated - did_control
 
-    return tau
+    return float(tau)
 
 
 def compute_placebo_effects(
@@ -971,4 +1311,4 @@ def compute_placebo_effects(
 
         placebo_effects.append(placebo_tau)
 
-    return np.array(placebo_effects)
+    return np.asarray(placebo_effects)
